/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AggregationMappingProvider } from '@admin-client/reporting-shared';
import { ApiRootModule } from '@alfa-client/api-root-shared';
import { EnvironmentModule, loadEnvironment } from '@alfa-client/environment-shared';
import { NavigationSharedModule } from '@alfa-client/navigation-shared';
import { registerLocaleData } from '@angular/common';
import { HTTP_INTERCEPTORS, provideHttpClient, withInterceptorsFromDi } from '@angular/common/http';
import localeDe from '@angular/common/locales/de';
import { enableProdMode, importProvidersFrom, Injectable } from '@angular/core';
import { bootstrapApplication, BrowserModule } from '@angular/platform-browser';
import { provideAnimations } from '@angular/platform-browser/animations';
import {
  ActivatedRouteSnapshot,
  BaseRouteReuseStrategy,
  provideRouter,
  RouteReuseStrategy,
  withRouterConfig,
} from '@angular/router';
import { HttpUnauthorizedInterceptor } from '@authentication';
import { EffectsModule } from '@ngrx/effects';
import { StoreRouterConnectingModule } from '@ngrx/router-store';
import { StoreModule } from '@ngrx/store';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { OAuthModule } from 'angular-oauth2-oidc';
import { provideAngularSvgIcon } from 'angular-svg-icon';
import { ConfigurationsProviders } from 'libs/admin/configuration-shared/src/lib/configuration.providers';
import { PostfachProviders } from 'libs/admin/postfach-shared/src/lib/postfach.providers';
import { SettingsProviders } from 'libs/admin/settings-shared/src/lib/settings.providers';
import { UserProviders } from 'libs/admin/user/src/lib/user.providers';
import { isNil } from 'lodash-es';
import { HttpBinaryFileInterceptor } from '../../../libs/tech-shared/src/lib/interceptor/http-binary-file.interceptor';
import { HttpXsrfInterceptor } from '../../../libs/tech-shared/src/lib/interceptor/http-xsrf.interceptor';
import { XhrInterceptor } from '../../../libs/tech-shared/src/lib/interceptor/xhr.interceptor';
import { AppComponent } from './app/app.component';
import { appRoutes } from './app/app.routes';
import { environment } from './environments/environment';

registerLocaleData(localeDe);

@Injectable()
export class NoRouteReuseStrategy extends BaseRouteReuseStrategy {
  public shouldReuseRoute(future: ActivatedRouteSnapshot, curr: ActivatedRouteSnapshot): boolean {
    return false;
  }
}

loadEnvironment(environment.environmentUrl).then((env) => {
  if (isNil(env?.production)) {
    console.error('Connection Error: environment is ', env);
    return;
  }

  if (env.production) {
    enableProdMode();
  }

  bootstrapApplication(AppComponent, {
    providers: [
      { provide: RouteReuseStrategy, useClass: NoRouteReuseStrategy },
      {
        provide: HTTP_INTERCEPTORS,
        useClass: XhrInterceptor,
        multi: true,
      },
      {
        provide: HTTP_INTERCEPTORS,
        useClass: HttpXsrfInterceptor,
        multi: true,
      },
      {
        provide: HTTP_INTERCEPTORS,
        useClass: HttpBinaryFileInterceptor,
        multi: true,
      },
      ConfigurationsProviders,
      PostfachProviders,
      SettingsProviders,
      UserProviders,
      AggregationMappingProvider,
      importProvidersFrom(
        NavigationSharedModule,
        BrowserModule,
        ApiRootModule,
        EnvironmentModule,
        environment.production ? [] : StoreDevtoolsModule.instrument({ connectInZone: true }),
        StoreModule.forRoot({}),
        EffectsModule.forRoot(),
        StoreRouterConnectingModule.forRoot(),
        OAuthModule.forRoot({
          resourceServer: {
            sendAccessToken: true,
          },
        }),
      ),
      provideAngularSvgIcon(),
      {
        provide: HTTP_INTERCEPTORS,
        useClass: HttpUnauthorizedInterceptor,
        multi: true,
      },
      provideHttpClient(withInterceptorsFromDi()),
      provideRouter(appRoutes, withRouterConfig({ onSameUrlNavigation: 'reload' })),
      provideAnimations(),
    ],
  }).catch((err) => console.error(err));
});
