import { describe, expect, it } from '@jest/globals';
import {
  ErrorRepresentation,
  KeycloakErrorMessage,
  KeycloakErrorResponseData,
  KeycloakFieldName,
  KeycloakHttpErrorResponse,
} from './keycloak-error.model';
import {
  canHandleKeycloakError,
  extractErrorRepresentations,
  isFieldErrorMessage,
  isSingleErrorMessage,
} from './keycloak-error.util';

describe('keycloak error model', () => {
  describe('extract error representations', () => {
    it('should throw error', () => {
      expect(() => extractErrorRepresentations({} as any)).toThrowError();
    });

    it('should return errors', () => {
      const keycloakError: KeycloakHttpErrorResponse = {
        response: {
          status: 400,
        },
        responseData: {
          errors: [
            {
              field: KeycloakFieldName.USERNAME,
              errorMessage: KeycloakErrorMessage.INVALID_LENGTH,
              params: ['username', 3, 255],
            },
            {
              field: KeycloakFieldName.EMAIL,
              errorMessage: KeycloakErrorMessage.INVALID_EMAIL,
              params: ['email', 'f'],
            },
          ],
        },
      };

      expect(extractErrorRepresentations(keycloakError)).toStrictEqual([
        {
          errorMessage: KeycloakErrorMessage.INVALID_LENGTH,
          params: ['username', 3, 255],
          field: KeycloakFieldName.USERNAME,
        } as ErrorRepresentation,

        {
          errorMessage: KeycloakErrorMessage.INVALID_EMAIL,
          params: ['email', 'f'],
          field: KeycloakFieldName.EMAIL,
        } as ErrorRepresentation,
      ]);
    });

    it('should error response data', () => {
      const keycloakError: KeycloakHttpErrorResponse = {
        response: {
          status: 400,
        },
        responseData: {
          field: KeycloakFieldName.USERNAME,
          errorMessage: KeycloakErrorMessage.INVALID_LENGTH,
          params: ['username', 3, 255],
        },
      };

      expect(extractErrorRepresentations(keycloakError)).toStrictEqual([
        {
          errorMessage: KeycloakErrorMessage.INVALID_LENGTH,
          params: ['username', 3, 255],
          field: KeycloakFieldName.USERNAME,
        } as ErrorRepresentation,
      ]);
    });
  });

  describe('can handle keycloak error', () => {
    const errorWithMessage: any = {
      response: {
        status: 400,
      },
      responseData: {
        errorMessage: 'User name is missing',
      },
    };

    const errorWithObject: any = {
      response: {
        status: 400,
      },
      responseData: {
        field: 'username',
        errorMessage: 'error-invalid-length',
        params: ['username', 3, 255],
      },
    };

    const errorWithArray: any = {
      response: {
        status: 400,
      },
      responseData: {
        errors: [
          {
            field: 'username',
            errorMessage: 'error-invalid-length',
            params: ['username', 3, 255],
          },
          {
            field: 'email',
            errorMessage: 'error-invalid-email',
            params: ['email', 'f'],
          },
        ],
      },
    };

    const errorWithoutErrorMessage: any = {
      response: {},
      responseData: {},
    };

    const errorWithoutResponseData: any = {
      response: {},
      responseData: {},
    };

    const error500: any = {
      response: {
        status: 500,
      },
      responseData: {
        errorMessage: 'internal error',
      },
    };

    it('should return true for text error message', () => {
      expect(canHandleKeycloakError(errorWithMessage)).toBe(true);
    });

    it('should return true for error object', () => {
      expect(canHandleKeycloakError(errorWithObject)).toBe(true);
    });

    it('should return true for error array', () => {
      expect(canHandleKeycloakError(errorWithArray)).toBe(true);
    });

    it('should return false for error without error message', () => {
      expect(canHandleKeycloakError(errorWithoutErrorMessage)).toBe(false);
    });

    it('should return false for error without response data', () => {
      expect(canHandleKeycloakError(errorWithoutResponseData)).toBe(false);
    });

    it('should return false for 500', () => {
      expect(canHandleKeycloakError(error500)).toBe(false);
    });

    describe('evaluate validation errors', () => {});
  });

  describe('isSingleErrorMessage', () => {
    it('should return false if contains field', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        field: KeycloakFieldName.USERNAME,
      };

      expect(isSingleErrorMessage(error)).toBe(false);
    });

    it('should return false if contains params', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        params: [],
      };

      expect(isSingleErrorMessage(error)).toBe(false);
    });

    it('should return false if contains errors', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        errors: [],
      };

      expect(isSingleErrorMessage(error)).toBe(false);
    });

    it('should return true', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
      };

      expect(isSingleErrorMessage(error)).toBe(true);
    });
  });

  describe('isFieldErrorMessage', () => {
    it('should return false if error message missing', () => {
      const error: KeycloakErrorResponseData = {
        params: [],
        field: KeycloakFieldName.USERNAME,
      };

      expect(isFieldErrorMessage(error)).toBe(false);
    });

    it('should return false if field missing', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        params: [],
      };

      expect(isFieldErrorMessage(error)).toBe(false);
    });

    it('should return false if params missing', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        field: KeycloakFieldName.USERNAME,
      };

      expect(isFieldErrorMessage(error)).toBe(false);
    });

    it('should return false if contains errors', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        field: KeycloakFieldName.USERNAME,
        params: [],
        errors: [],
      };

      expect(isFieldErrorMessage(error)).toBe(false);
    });

    it('should return true', () => {
      const error: KeycloakErrorResponseData = {
        errorMessage: KeycloakErrorMessage.USERNAME_IS_MISSING,
        field: KeycloakFieldName.USERNAME,
        params: [],
      };

      expect(isFieldErrorMessage(error)).toBe(true);
    });
  });
});
