/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiRootLinkRel, ApiRootResource } from '@alfa-client/api-root-shared';
import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { dispatchEventFromFixtureByType, existsAsHtmlElement, getElementComponentFromFixtureByCss, getElementFromFixture, mock, Mock, notExistsAsHtmlElement, } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { RouterTestingModule } from '@angular/router/testing';
import { AuthenticationService } from '@authentication';
import { expect } from '@jest/globals';
import { getUrl } from '@ngxp/rest';
import { DropdownMenuButtonItemComponent, DropdownMenuComponent, LogoutIconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { createApiRootResource } from '../../../../../api-root-shared/test/api-root';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { DocumentationComponent } from '../documentation/documentation.component';
import { AdminUserLogoutButtonComponent } from '../user-logout-button/admin-user-logout-button.component';
import { UserProfileButtonContainerComponent } from './user-profile.button-container.component';

describe('UserProfileButtonContainerComponent', () => {
  let component: UserProfileButtonContainerComponent;
  let fixture: ComponentFixture<UserProfileButtonContainerComponent>;

  let authenticationService: Mock<AuthenticationService>;

  const popupButtonContent: string = getDataTestIdOf('popup-button-content');
  const popupLogoutButton: string = getDataTestIdOf('popup-logout-button');
  const documentationTestId: string = getDataTestIdOf('admin-documentation');

  const apiRootStateResource: StateResource<ApiRootResource> = createStateResource(createApiRootResource());

  beforeEach(() => {
    authenticationService = mock(AuthenticationService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [UserProfileButtonContainerComponent],
      imports: [
        RouterTestingModule,
        MockComponent(DropdownMenuComponent),
        MockComponent(DropdownMenuButtonItemComponent),
        MockComponent(LogoutIconComponent),
      ],
      providers: [
        {
          provide: AuthenticationService,
          useValue: authenticationService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(UserProfileButtonContainerComponent);
    component = fixture.componentInstance;
    component.apiRootStateResource = apiRootStateResource;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call authService to get current user initials', () => {
      component.ngOnInit();

      expect(authenticationService.getCurrentUserInitials).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('popup button', () => {
      it('should show initials', () => {
        component.currentUserInitials = 'AV';
        fixture.detectChanges();

        const popupButtonContentElement: HTMLElement = getElementFromFixture(fixture, popupButtonContent);

        expect(popupButtonContentElement.textContent.trim()).toEqual('AV');
      });
    });

    describe('logout', () => {
      it('should call authService logout', () => {
        dispatchEventFromFixtureByType(fixture, AdminUserLogoutButtonComponent, 'logout');

        expect(authenticationService.logout).toHaveBeenCalled();
      });
    });

    describe('documentation', () => {
      it('should exists', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.DOCUMENTATIONS]));

        fixture.detectChanges();

        existsAsHtmlElement(fixture, documentationTestId);
      });

      it('should NOT exists', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource([]));

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, documentationTestId);
      });

      it('should have inputs', () => {
        component.apiRootStateResource = createStateResource(createApiRootResource([ApiRootLinkRel.DOCUMENTATIONS]));

        fixture.detectChanges();
        const documentationComponent: DocumentationComponent = getElementComponentFromFixtureByCss(fixture, documentationTestId);

        expect(documentationComponent.url).toEqual(
          getUrl(component.apiRootStateResource.resource, ApiRootLinkRel.DOCUMENTATIONS),
        );
      });
    });
  });
});
