/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  Bescheid,
  BESCHEID_UPLOADED_ATTACHMENTS,
  BescheidDocument,
  BescheidLinkRel,
  BescheidListResource,
  BescheidResource,
  BescheidSendBy,
  BescheidStatus,
  BescheidWizardStep,
  buildCreateBescheidCommand,
  buildCreateBescheidDocumentCommandProps,
  buildCreateBescheidDocumentFromFileProps,
  buildSendBescheidCommandProps,
  buildUpdateBescheidCommandProps,
  createBescheidListResourceService,
  createBescheidResourceService,
  createEmptyBescheidDocument,
  createEmptyUploadInProgress,
  createInitialWizard,
  DocumentResource,
  Wizard,
} from '@alfa-client/bescheid-shared';
import {
  BinaryFileListLinkRel,
  BinaryFileListResource,
  BinaryFileResource,
  BinaryFileService,
} from '@alfa-client/binary-file-shared';
import {
  CommandOrder,
  CommandResource,
  CommandService,
  getEffectedResourceUrl,
  notHasCommandError,
  tapOnCommandSuccessfullyDone,
} from '@alfa-client/command-shared';
import { PostfachService } from '@alfa-client/postfach-shared';
import {
  createEmptyStateResource,
  createStateResource,
  filterIsLoadedOrHasError,
  getEmbeddedResources,
  hasStateResourceError,
  isLoaded,
  isNotEmpty,
  isNotNil,
  ResourceRepository,
  sortByGermanDateStr,
  StateResource,
} from '@alfa-client/tech-shared';
import { VorgangCommandService, VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { inject, Injectable } from '@angular/core';
import { getUrl, hasLink, LinkRel, Resource, ResourceUri } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { BehaviorSubject, filter, first, map, Observable, startWith, switchMap } from 'rxjs';
import { BescheidFacade } from './+state/bescheid.facade';
import { DocumentLinkRel } from './document.linkrel';

@Injectable({ providedIn: 'root' })
export class BescheidService {
  private readonly bescheidFacade = inject(BescheidFacade);
  private readonly commandService = inject(CommandService);
  private readonly vorgangCommandService = inject(VorgangCommandService);
  private readonly binaryFileService = inject(BinaryFileService);
  private readonly resourceRepository = inject(ResourceRepository);
  private readonly postfachService = inject(PostfachService);
  private readonly vorgangService = inject(VorgangService);
  // The injection of this services is problematic.
  // It must be analysed how to inject BescheidService via providers in BescheidSharedModule.
  private readonly bescheidResourceService = createBescheidResourceService(
    this.resourceRepository,
    this.commandService,
    this.vorgangService,
  );
  private readonly bescheidListResourceService = createBescheidListResourceService(this.resourceRepository, this.vorgangService);
  //
  readonly _bescheidDocument$: BehaviorSubject<BescheidDocument> = new BehaviorSubject(createEmptyBescheidDocument());
  readonly _wizard$: BehaviorSubject<Wizard> = new BehaviorSubject(createInitialWizard());

  public init(): void {
    this._wizard$.next(createInitialWizard());
    this._bescheidDocument$.next(createEmptyBescheidDocument());
  }

  public exit(): void {
    this.bescheidListResourceService.refresh();
    this.postfachService._setPostfachMailOnReload();
    this._clearUploadedFiles();
  }

  public skipBescheidCreation(
    vorgangWithEingangResource: VorgangWithEingangResource,
    bescheidResource: BescheidResource,
  ): Observable<StateResource<CommandResource>> {
    if (isNil(bescheidResource)) {
      return this.vorgangCommandService.abschliessen(vorgangWithEingangResource);
    }
    return this.deleteBescheidAndCompleteVorgang(vorgangWithEingangResource);
  }

  deleteBescheidAndCompleteVorgang(
    vorgangWithEingangResource: VorgangWithEingangResource,
  ): Observable<StateResource<CommandResource>> {
    return this.vorgangCommandService
      .abschliessen(vorgangWithEingangResource)
      .pipe(tapOnCommandSuccessfullyDone(() => this.deleteBescheid()));
  }

  public deleteBescheid(): Observable<StateResource<CommandResource>> {
    return this.bescheidResourceService.delete();
  }

  public loadFiles(bescheidResource: BescheidResource): void {
    this.loadBescheidDocument(bescheidResource);
    this.loadAttachments(bescheidResource);
  }

  public createBescheidDocument(bescheidResource: BescheidResource): void {
    this._bescheidDocument$.next({ ...createEmptyBescheidDocument(), create: createEmptyStateResource(true) });
    this.doCreateBescheidDocument(bescheidResource)
      .pipe(filterIsLoadedOrHasError(), first())
      .subscribe((commandStateResource: StateResource<CommandResource>) =>
        this.handleCreateBescheidDocumentResponse(commandStateResource),
      );
  }

  doCreateBescheidDocument(bescheidResource: BescheidResource): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps(buildCreateBescheidDocumentCommandProps(bescheidResource));
  }

  handleCreateBescheidDocumentResponse(commandStateResource: StateResource<CommandResource>): void {
    if (notHasCommandError(commandStateResource.resource)) {
      const documentUri: ResourceUri = getEffectedResourceUrl(commandStateResource.resource);
      this._bescheidDocument$.next({ ...this._bescheidDocument$.value, documentUri });
      this.loadBescheidDocumentByUri(documentUri);
    } else {
      this._bescheidDocument$.next({ ...this._bescheidDocument$.value, create: commandStateResource });
    }
  }

  emitBescheidDocumentError(stateResource: StateResource<Resource>): void {
    const value: BescheidDocument = this._bescheidDocument$.value;
    this._bescheidDocument$.next({
      ...value,
      upload: { ...value.upload, loading: false, error: stateResource.error },
    });
  }

  loadBescheidDocument(bescheidResource: BescheidResource): void {
    if (hasLink(bescheidResource, BescheidLinkRel.BESCHEID_DOCUMENT)) {
      this.loadBescheidDocumentByUri(getUrl(bescheidResource, BescheidLinkRel.BESCHEID_DOCUMENT));
    }
  }

  public loadBescheidDocumentByUri(resourceUri: ResourceUri): void {
    this.resourceRepository
      .getResource(resourceUri)
      .pipe(first())
      .subscribe((document: DocumentResource) => this.loadBescheidDocumentFile(document));
  }

  loadBescheidDocumentFile(document: DocumentResource): void {
    this.binaryFileService
      .getFile(getUrl(document, DocumentLinkRel.FILE))
      .pipe(filterIsLoadedOrHasError(), first())
      .subscribe((binaryFile: StateResource<BinaryFileResource>) => {
        this._bescheidDocument$.next({
          ...this._bescheidDocument$.value,
          upload: createEmptyUploadInProgress(),
          create: createEmptyStateResource(),
          resource: binaryFile.resource,
          documentUri: getUrl(document, LinkRel.Self),
        });
      });
  }

  loadAttachments(bescheidResource: BescheidResource): void {
    if (hasLink(bescheidResource, BescheidLinkRel.ATTACHMENTS)) {
      this.binaryFileService
        .getFiles(bescheidResource, BescheidLinkRel.ATTACHMENTS)
        .pipe(
          filterIsLoadedOrHasError(),
          first(),
          map((stateResource: StateResource<BinaryFileListResource>) =>
            getEmbeddedResources<BinaryFileResource>(stateResource, BinaryFileListLinkRel.FILE_LIST),
          ),
        )
        .subscribe((files: BinaryFileResource[]) => this.binaryFileService.addFiles(BESCHEID_UPLOADED_ATTACHMENTS, files));
    }
  }

  public uploadBescheidDocument(document: File, bescheid: BescheidResource): void {
    this._bescheidDocument$.next({ ...this._bescheidDocument$.value, upload: { fileName: document.name, loading: true } });
    this.binaryFileService
      .uploadFile(bescheid, BescheidLinkRel.UPLOAD_BESCHEID_FILE, document, false)
      .pipe(filterIsLoadedOrHasError(), first())
      .subscribe((binaryFileStateResource: StateResource<BinaryFileResource>) =>
        this.handleUploadBescheidDocumentResponse(bescheid, binaryFileStateResource),
      );
  }

  handleUploadBescheidDocumentResponse(
    bescheid: BescheidResource,
    binaryFileStateResource: StateResource<BinaryFileResource>,
  ): void {
    if (hasStateResourceError(binaryFileStateResource)) {
      this.emitBescheidDocumentError(binaryFileStateResource);
    } else {
      this.createBescheidDocumentFromFile(bescheid, binaryFileStateResource.resource);
    }
  }

  createBescheidDocumentFromFile(bescheid: BescheidResource, binaryFile: BinaryFileResource): void {
    this.commandService
      .createCommandByProps(buildCreateBescheidDocumentFromFileProps(bescheid, binaryFile))
      .pipe(filterIsLoadedOrHasError(), first())
      .subscribe((commandStateResource: StateResource<CommandResource>) =>
        this.handleCreateBescheidDocumentFromFileResponse(commandStateResource, binaryFile),
      );
  }

  handleCreateBescheidDocumentFromFileResponse(
    commandStateResource: StateResource<CommandResource>,
    binaryFile: BinaryFileResource,
  ): void {
    if (hasStateResourceError(commandStateResource)) {
      this.emitBescheidDocumentError(commandStateResource);
    } else {
      this._bescheidDocument$.next({
        ...this._bescheidDocument$.value,
        documentUri: getEffectedResourceUrl(commandStateResource.resource),
        upload: createEmptyUploadInProgress(),
        resource: binaryFile,
      });
    }
  }

  public createBescheid(
    vorgangWithEingang: VorgangWithEingangResource,
    bescheid?: Bescheid,
  ): Observable<StateResource<CommandResource>> {
    this.bescheidFacade.createBescheidDraft(vorgangWithEingang, buildCreateBescheidCommand(bescheid));
    return this.bescheidFacade
      .getBescheidCommand()
      .pipe(
        tapOnCommandSuccessfullyDone((commandStateResource: StateResource<CommandResource>) =>
          this.updateBescheidDraft(commandStateResource.resource),
        ),
      );
  }

  updateBescheidDraft(command: CommandResource): void {
    this.bescheidResourceService.loadByResourceUri(getEffectedResourceUrl(command));
    if (command.order === CommandOrder.CREATE_BESCHEID) {
      this._wizard$.next({ ...this._wizard$.value, bescheidCreated: true });
    }
  }

  public updateBescheid(
    bescheidResource: BescheidResource,
    updatedBescheid: Bescheid,
  ): Observable<StateResource<CommandResource>> {
    return this.doUpdateBescheid(bescheidResource, updatedBescheid).pipe(
      tapOnCommandSuccessfullyDone((commandStateResource: StateResource<CommandResource>) => {
        this.updateBescheidDraft(commandStateResource.resource);
      }),
    );
  }

  doUpdateBescheid(bescheidResource: BescheidResource, bescheid: Bescheid): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps(buildUpdateBescheidCommandProps(bescheidResource, bescheid));
  }

  public sendBescheidManually(): Observable<StateResource<CommandResource>> {
    return this.sendBescheid(BescheidLinkRel.BESCHEIDEN);
  }

  public sendBescheidMessage(): Observable<StateResource<CommandResource>> {
    return this.sendBescheid(BescheidLinkRel.BESCHEIDEN_UND_SENDEN);
  }

  sendBescheid(linkRel: BescheidLinkRel): Observable<StateResource<CommandResource>> {
    return this.getBescheidDraft().pipe(
      filter(isLoaded),
      map((stateResource: StateResource<BescheidResource>) => stateResource.resource),
      switchMap((bescheidResource: BescheidResource) =>
        this.commandService.createCommandByProps(buildSendBescheidCommandProps(bescheidResource, linkRel)),
      ),
    );
  }

  public getBescheidDraft(): Observable<StateResource<BescheidResource>> {
    return this.bescheidResourceService.get();
  }

  public getBescheidDocument(): Observable<BescheidDocument> {
    return this._bescheidDocument$.asObservable();
  }

  public deleteBescheidDocument(): void {
    this._bescheidDocument$.next(createEmptyBescheidDocument());
  }

  public getActiveStep(): Observable<BescheidWizardStep> {
    return this._wizard$.asObservable().pipe(map((wizard: Wizard) => wizard.activeStep));
  }

  public setActiveStep(step: BescheidWizardStep): void {
    this._clearUploadedFiles();
    this._wizard$.next({ ...this._wizard$.value, activeStep: step });
  }

  public getBescheidCreated(): Observable<boolean> {
    return this._wizard$.asObservable().pipe(map((wizard: Wizard) => wizard.bescheidCreated));
  }

  public selectBescheidResource(): Observable<StateResource<BescheidResource>> {
    return this.bescheidResourceService.selectResource();
  }

  public getWizard(): Observable<Wizard> {
    return this._wizard$.asObservable();
  }

  public setSendBy(sendBy: BescheidSendBy): void {
    this._wizard$.next({ ...this._wizard$.value, sendBy: sendBy });
  }

  public setNachrichtEmpfaenger(empfaenger: string): void {
    this._wizard$.next({ ...this._wizard$.value, empfaenger: empfaenger });
  }

  public lockBescheidSending(): void {
    this._wizard$.next({ ...this._wizard$.value, canBeSend: false });
  }

  public unlockBescheidSending(): void {
    this._wizard$.next({ ...this._wizard$.value, canBeSend: true });
  }

  public finishAddingBescheidDocument(): void {
    this._bescheidDocument$.next({
      ...this._bescheidDocument$.value,
      upload: createEmptyUploadInProgress(),
      create: createEmptyStateResource(),
    });
  }

  _clearUploadedFiles(): void {
    this.binaryFileService.clearUploadedFiles(BESCHEID_UPLOADED_ATTACHMENTS);
  }

  public getBescheidList(): Observable<StateResource<BescheidListResource>> {
    return this.bescheidListResourceService.getList();
  }

  public getBescheidDocumentByUri(resourceUri: ResourceUri): Observable<StateResource<DocumentResource>> {
    return this.resourceRepository.getResource<DocumentResource>(resourceUri).pipe(
      map((documentResource: DocumentResource) => createStateResource(documentResource)),
      startWith(createEmptyStateResource<DocumentResource>(true)),
    );
  }

  public getLastBescheid(): Observable<BescheidResource> {
    return this.bescheidListResourceService.getItems().pipe(
      map((bescheide: BescheidResource[]) => this.filterBySentStatus(bescheide)),
      map((bescheide: BescheidResource[]) => this.sortByBeschiedenAm(bescheide)),
      map((bescheide: BescheidResource[]) => bescheide[0]),
    );
  }

  filterBySentStatus(bescheide: BescheidResource[]): BescheidResource[] {
    return isNotNil(bescheide) ? bescheide.filter(this.hasSentStatus) : [];
  }

  private hasSentStatus(bescheid: BescheidResource): boolean {
    return bescheid.status === BescheidStatus.SENT;
  }

  private sortByBeschiedenAm(bescheide: BescheidResource[]): BescheidResource[] {
    return sortByGermanDateStr<BescheidResource>(bescheide, (bescheid: BescheidResource) => bescheid.beschiedenAm);
  }

  public existBescheid(): Observable<boolean> {
    return this.bescheidListResourceService.getItems().pipe(
      map((bescheide: BescheidResource[]) => this.filterBySentStatus(bescheide)),
      map((bescheide: BescheidResource[]) => isNotEmpty(bescheide)),
    );
  }
}
